  --[[Artifact Tracker - Track your progress towards additional colors for your hidden artifact!
    Copyright (C) 2016 Allen Underwood

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.--]]

local Player_Name = UnitName("player")
local Player_Realm = GetRealmName()
local g,s,p,d=GetAchievementCriteriaInfo,select,print,0 
local Dungeons=0
local WQs=0
local Kills=0
local TrackerCheckButtons = {}
local TrackerFrame = CreateFrame("Frame", "TrackerFrame", UIParent)
local FrameTex = TrackerFrame:CreateTexture("Interface\DialogFrame\UI-DialogBox-Border")
local TrackerText = TrackerFrame:CreateFontString("Progress", "OVERLAY", "GameFontWhite")
local ReportTo = nil

-- SavedVariablesPerCharacter
Tracker_Settings = {
	MinimapPos = 45; -- default position of the minimap icon in degrees
	CheckedBox = 1; -- default checked box
	ToggleFrame = false; --default setting for on screen frame
}

-- CREDIT TO GELLO-UNDERHILL
-- returns the parameters to follow text on a SendChatMessage to use the last used chat method (sticky chat)
function StickyParameters()

  local editBox = ChatFrame1EditBox
  local stickyType = editBox:GetAttribute("stickyType")

  if stickyType=="WHISPER" then
    -- for whispers use tellTarget attribute for name
    return "WHISPER",nil,editBox:GetAttribute("tellTarget")
  elseif stickyType=="CHANNEL" then
    -- for channel chats reuse last-used channelTarget
    return "CHANNEL",nil,editBox:GetAttribute("channelTarget")
  elseif not stickyType then
    -- if no chats send yet, use default /say
    return "SAY"
  else
    -- for everything else ("PARTY","GUILD",etc) use the sticky type
    return stickyType
  end
end

--Sets up frame on addon load
function Tracker_OnLoad(self)
	self:RegisterForDrag("LeftButton");  

			-- Set up the on screen frame
	TrackerFrame:SetMovable(true)
	TrackerFrame:EnableMouse(true)
	TrackerFrame:RegisterForDrag("LeftButton")
	TrackerFrame:SetScript("OnDragStart", TrackerFrame.StartMoving)
	TrackerFrame:SetScript("OnDragStop", TrackerFrame.StopMovingOrSizing)
	TrackerFrame:RegisterEvent("QUEST_TURNED_IN")
	TrackerFrame:RegisterEvent("PLAYER_PVP_KILLS_CHANGED")
	TrackerFrame:RegisterEvent("ENCOUNTER_END")
	TrackerFrame:RegisterEvent("ADDON_LOADED")

	TrackerFrame:SetScript("OnEvent", TrackerEvent)

	TrackerFrame:SetPoint("CENTER")
	TrackerFrame:SetSize(108, 48)
	FrameTex:SetAllPoints()
	FrameTex:SetColorTexture(0.0, 0.0, 0.0, 0.7)
	TrackerText:SetPoint("CENTER")
	TrackerText:SetFontObject("GameFontWhiteSmall")
end

-- move the minimap button to its saved position, should be after variables loaded or later
function Tracker_MinimapButton_Reposition()
	Tracker_MinimapButton:SetPoint("TOPLEFT","Minimap","TOPLEFT",52-(80*cos(Tracker_Settings.MinimapPos)),(80*sin(Tracker_Settings.MinimapPos))-52)
end

-- Only while the button is dragged this is called every frame
function Tracker_MinimapButton_DraggingFrame_OnUpdate()

	local xpos,ypos = GetCursorPosition()
	local xmin,ymin = Minimap:GetLeft(), Minimap:GetBottom()

	xpos = xmin-xpos/UIParent:GetScale()+70 -- get coordinates as differences from the center of the minimap
	ypos = ypos/UIParent:GetScale()-ymin-70

	Tracker_Settings.MinimapPos = math.deg(math.atan2(ypos,xpos)) -- save the degrees we are relative to the minimap center
	Tracker_MinimapButton_Reposition() -- move the button
end

--Credit to the guys at the following link for the functions to track progress: https://eu.battle.net/forums/en/wow/topic/17613332580#post-1
--Get achievement progress information and set it to variables
function Tracker_SetInfo()
	d=0
	for i = 1, GetAchievementNumCriteria(11152)
		do d = d + s(4, g(11152, i))
	end

	if d > 30 then
		d = 30
	end

	WQs = "WQs: "..s(4, g(11153, 1)).."/"..s(5, g(11153, 1))
	Kills = "Kills: "..s(4, g(11154, 1)).."/"..s(5, g(11154, 1))
	Dungeons = "Dungeons: " ..d.."/"..s(5, g(11152, 1))
end

--Toggles the display frame option
function FrameToggle()
	if FrameToggleButton:GetChecked() == true then
		TrackerFrame:Show()
		Tracker_Settings.ToggleFrame = true
		FrameToggleButton:SetChecked(true)
	else
		TrackerFrame:Hide()
		Tracker_Settings.ToggleFrame = false
		FrameToggleButton:SetChecked(false)
	end
end

--Toggler of left click of the minimap icon
function Frame_Toggle()
	if FrameToggleButton:GetChecked() == true then
		FrameToggleButton:SetChecked(false)
	else
		FrameToggleButton:SetChecked(true)
	end
	FrameToggle()
end

--Display the tooltip for the minimap when moused over
function Tracker_DisplayToolTip()
	GameTooltip:SetOwner(Tracker_MinimapButton, "ANCHOR_RIGHT")
	GameTooltip:SetText(WQs.."\r".. Kills .. "\r" .. Dungeons .. "\rRight-Click for Options")
	GameTooltip:Show()
end

--Set all reporting buttons in the correct manner
function Tracker_OnShow()
	SetButtons(Tracker_Settings.CheckedBox);
end

function SetButtons(boxNumber)
	for i = 1, table.maxn(TrackerCheckButtons)
		do TrackerCheckButtons[i]:SetChecked(false)
	end
	TrackerCheckButtons[boxNumber]:SetChecked(true)
	Tracker_Settings.CheckedBox = boxNumber
end
--Create all the check box buttons for the options menu.
function CreateCheckbuttons()  --)

--Frame Toggle Check Button
FrameToggleButton = CreateFrame("CheckButton", "FrameCheckButton_GlobalName", Tracker_Frame, "ChatConfigCheckButtonTemplate");
FrameToggleButton:SetPoint("TOPLEFT", 15, -45);
FrameCheckButton_GlobalNameText:SetText("Toggle Frame");
FrameToggleButton:SetScript("OnClick", 
  function()
  	FrameToggle();
  end
 );

--Self Report Only Butto
TrackerCheckButtons[1] = CreateFrame("CheckButton", "SelfReportButton_GlobalName", Tracker_Frame, "ChatConfigCheckButtonTemplate");
TrackerCheckButtons[1]:SetPoint("TOPLEFT", 15, -85);
SelfReportButton_GlobalNameText:SetText("Self Only");
TrackerCheckButtons[1]:SetScript("OnClick", 
  function()
  	SetButtons(1)
  end
);

--Default Chat Report Only Button
TrackerCheckButtons[2] = CreateFrame("CheckButton", "DefaultReportButton_GlobalName", Tracker_Frame, "ChatConfigCheckButtonTemplate");
TrackerCheckButtons[2]:SetPoint("TOPLEFT", 15, -105);
DefaultReportButton_GlobalNameText:SetText("Default Chat");
TrackerCheckButtons[2]:SetScript("OnClick", 
  function()
  	SetButtons(2)
  end
);

--Say Report Only Button
TrackerCheckButtons[3] = CreateFrame("CheckButton", "SayReportButton_GlobalName", Tracker_Frame, "ChatConfigCheckButtonTemplate");
TrackerCheckButtons[3]:SetPoint("TOPLEFT", 15, -125);
SayReportButton_GlobalNameText:SetText("Say");
TrackerCheckButtons[3]:SetScript("OnClick", 
  function()
  	SetButtons(3)
  end
);

--Party Report Only Button
TrackerCheckButtons[4] = CreateFrame("CheckButton", "PartyReportButton_GlobalName", Tracker_Frame, "ChatConfigCheckButtonTemplate");
TrackerCheckButtons[4]:SetPoint("TOPLEFT", 15, -145);
PartyReportButton_GlobalNameText:SetText("Party");
TrackerCheckButtons[4]:SetScript("OnClick", 
  function()
  	SetButtons(4)
  end
);

--Raid Report Only Button
TrackerCheckButtons[5] = CreateFrame("CheckButton", "RaidReportButton_GlobalName", Tracker_Frame, "ChatConfigCheckButtonTemplate");
TrackerCheckButtons[5]:SetPoint("TOPLEFT", 15, -165);
RaidReportButton_GlobalNameText:SetText("Raid");
TrackerCheckButtons[5]:SetScript("OnClick", 
  function()
  	SetButtons(5)
  end
);

--Guild Report Only Button
TrackerCheckButtons[6] = CreateFrame("CheckButton", "GuildReportButton_GlobalName", Tracker_Frame, "ChatConfigCheckButtonTemplate");
TrackerCheckButtons[6]:SetPoint("TOPLEFT", 15, -185);
GuildReportButton_GlobalNameText:SetText("Guild");
TrackerCheckButtons[6]:SetScript("OnClick", 
  function()
  	SetButtons(6)
  end
);

end

--Reports to specified location based on options menu
function Tracker_MinimapButton_OnReportClick()
	if Tracker_Settings.CheckedBox == 1 then --Self only
		print(Player_Name.."-" ..Player_Realm .. " Hidden Artifact Weapon Progress:")
		print(WQs)
		print(Kills)
		print(Dungeons)
	elseif Tracker_Settings.CheckedBox == 2 then --Default chat
		SendChatMessage(Player_Name.."-" ..Player_Realm .. " Hidden Artifact Weapon Progress:", StickyParameters())
		SendChatMessage(WQs, StickyParameters())
		SendChatMessage(Kills, StickyParameters())
		SendChatMessage(Dungeons, StickyParameters())
	elseif Tracker_Settings.CheckedBox == 3 then -- Say Chat
		ReportProgress("SAY")
	elseif Tracker_Settings.CheckedBox == 4 then --Party Chat
		ReportProgress("PARTY")
	elseif Tracker_Settings.CheckedBox == 5 then --Raid Chat
		ReportProgress("RAID")
	elseif Tracker_Settings.CheckedBox == 6 then --guild Chat
		ReportProgress("GUILD")
	end
end

--Helper function for reporting
function ReportProgress(ReportTo)
	SendChatMessage(Player_Name.."-" ..Player_Realm .. " Hidden Artifact Weapon Progress:", ReportTo)
	SendChatMessage(WQs, ReportTo)
	SendChatMessage(Kills, ReportTo)
	SendChatMessage(Dungeons, ReportTo)
end

--Triggered when the events are fired to reset information and keep the displays updated
function TrackerEvent()
	FrameToggleButton:SetChecked(Tracker_Settings.ToggleFrame)

	if Tracker_Settings.ToggleFrame == true then
		TrackerFrame:Show()
	else
		TrackerFrame:Hide()
	end

	Tracker_SetInfo()
	TrackerText:SetText(WQs .. "\r" .. Kills.. "\r" .. Dungeons)
end
